-- --------------------------------------------------
-- Innkeep Export
-- --------------------------------------------------
InnkeepExport = {}

function InnkeepExport:Timestamp()
  return date("%Y-%m-%d %H:%M")
end

function InnkeepExport:Create(type, data)
  return {
    metadata = self:CreateMetadata(type),
    data = data,
  }
end

function InnkeepExport:UnitData(unit)
  return {
    faction = UnitFactionGroup(unit),
    name = GetUnitName(unit),
    race = UnitRace(unit),
    class = UnitClass(unit),
    guild = GetGuildInfo(unit),
  }
end

function InnkeepExport:CreateMetadata(type)
  return {
    version = "1.0.0",
    timestamp = self:Timestamp(),
    realm = GetRealmName(),
    character = self:UnitData("player"),
    type = type,
  }
end

function InnkeepExport:encode(data)
  return Base64:encode(JSON.encode(data))
end

function InnkeepExport:decode(data)
  return JSON.decode(Base64:decode(data))
end

-- --------------------------------------------------
-- Innkeep GUI
-- --------------------------------------------------
AceGUI = LibStub("AceGUI-3.0")
InnkeepGUI = {}

function InnkeepGUI:ShowExport(data)
  local f = AceGUI:Create("Frame")
  f:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
  f:SetTitle("Innkeep Export")
  f:SetStatusText("Upload to: https://innkeep.app")
  f:SetLayout("Fill")

  local editbox = AceGUI:Create("MultiLineEditBox")
  editbox:SetLabel(data.metadata.type:gsub("^%l", string.upper))
  editbox:SetText(InnkeepExport:encode(data))
  editbox:DisableButton(true)
  f:AddChild(editbox)

  editbox:HighlightText()
end
